<?php
/**
 * CryptoPhoto Special Settings page
 * 
 * @file
 * @ingroup Extensions
 */

class SpecialCryptoPhotoSettings extends SpecialPage {
    function __construct() {
        parent::__construct( 'CryptoPhotoSettings', '', false );
    }

    function execute( $par ) {
        global $wgUser, $mediaWiki, $wgRequest, $wgOut, $wgCryptoPhotoConfig, $IP, $wgServer, $wgScriptPath, $wgSecretKey;

        CryptoPhotoDatabaseSupport::loadSettings();

        // get the WebRequest object
        $req = $this->getRequest();
    
        // check if the user wants to test the CryptoPhoto setup (this is an AJAX request)
        if ($req->getVal('test') !== null) {
            return $this->test();
        }
    
        // prepare the page
        $this->setHeaders();
        $this->outputHeader();

        //check for authenticated admin user
        if ($wgUser->isLoggedIn() && in_array('sysop', $wgUser->getEffectiveGroups())) {
            // get all user groups
            $config = $this->getConfig();
            $groupPermissions = $config->get('GroupPermissions');
            $revokePermissions = $config->get('RevokePermissions');
            $addGroups = $config->get('AddGroups');
            $removeGroups = $config->get('RemoveGroups');
            $groupsAddToSelf = $config->get('GroupsAddToSelf');
            $groupsRemoveFromSelf = $config->get('GroupsRemoveFromSelf');

            // put them together
            $allGroups = array_unique(array_merge(
                array_keys($groupPermissions),
                array_keys($revokePermissions),
                array_keys($addGroups),
                array_keys($removeGroups),
                array_keys($groupsAddToSelf),
                array_keys($groupsRemoveFromSelf)
            ));

            // and sort them
            asort($allGroups);
      
            // this will hold the choices for the groups selection
            $groupChoices = array();
      
            // add groups and groups names
            foreach ($allGroups as $group) {
                if ($group == '*') continue;
                $msg = $this->msg('group-' . $group);
                $groupChoices[!$msg->isBlank() ? $msg->text() : $group] = $group;
            }
      
            // get the OuputPage object
            $out = $this->getOutput();
      
            // CryptoPhoto autoconfiguration wizzard
            $wizzard = '<p>' . "\n" .
                            wfMessage('cpsettings-autoconf')->text() . "\n" .
                            '<input name="Wizard" type="button" onclick="return CPPluginWizard.start_wizard()" value="' . wfMessage('cpsettings-wizzard-button')->text() . '" />' . "\n" .
                       '</p>';

            // CryptoPhoto Settings form definition
            $form = array(
                'publickey' => array(
                    'section' => 'ks',
                    'class' => 'HTMLTextField',
                    'label-message' => 'cpsettings-publickey-label',
                    'help-message' => 'cpsettings-publickey-help',
                    'id' => 'publickey',
                    'name' => 'publickey',
                    'required' => true,
                    'default' => $wgCryptoPhotoConfig['publickey']
                ),
                'privatekey' => array(
                    'section' => 'ks',
                    'class' => 'HTMLTextField',
                    'label-message' => 'cpsettings-privatekey-label',
                    'help-message' => 'cpsettings-privatekey-help',
                    'id' => 'privatekey',
                    'name' => 'privatekey',
                    'required' => true,
                    'default' => $wgCryptoPhotoConfig['privatekey']
                ),
                'salt' => array(
                    'section' => 'ks',
                    'class' => 'HTMLTextField',
                    'label-message' => 'cpsettings-salt-label',
                    'help-message' => 'cpsettings-salt-help',
                    'id' => 'salt',
                    'name' => 'salt',
                    'required' => true,
                    'default' => $wgCryptoPhotoConfig['salt']
                ),
                'server' => array(
                    'section' => 'ks',
                    'class' => 'HTMLTextField',
                    'label-message' => 'cpsettings-server-label',
                    'help-message' => 'cpsettings-server-help',
                    'id' => 'server',
                    'name' => 'server',
                    'required' => true,
                    'default' => $wgCryptoPhotoConfig['server']
                ),
                'mandatory_groups' => array(
                    'section' => 'options',
                    'type' => 'multiselect',
                    'label-message' => 'cpsettings-mandatory-label',
                    'help-message' => 'cpsettings-mandatory-help',
                    'id' => 'mandatory_groups',
                    'name' => 'mandatory_groups',
                    'options' => $groupChoices,
                    'default' => $wgCryptoPhotoConfig['mandatory_groups']
                ),
                'groups' => array(
                    'section' => 'options',
                    'type' => 'multiselect',
                    'label-message' => 'cpsettings-allowed-label',
                    'help-message' => 'cpsettings-allowed-help',
                    'id' => 'groups',
                    'name' => 'groups',
                    'options' => $groupChoices,
                    'default' => ($wgCryptoPhotoConfig['setup_complete'] ? $wgCryptoPhotoConfig['groups'] : array_values($groupChoices))
                ),
                'testresult' => array(
                    'type' => 'info',
                    'default' => '<tr><td colspan="2"><div id="testresult" style="display:none"></div></td></tr>',
                    'rawrow' => true
                )
            );
      
            // create the form object
            $htmlForm = new HTMLForm($form, $this->getContext(), 'cpsettings');
            
            // add autoconfiguration wizzard
            $htmlForm->addHeaderText($wizzard);
            
            // form buttons and callbacks
            $htmlForm->addButton('test_config', wfMessage('cpsettings-test-button')->text(), 'test_config', array('onClick' => 'testConfig(); return false;', 'type' => 'button'));
            $htmlForm->setSubmitText(wfMessage('cpsettings-save-button')->text());
            $htmlForm->setSubmitCallback(array('SpecialCryptoPhotoSettings', 'save'));

            // prepare the from to be displayed
            $htmlForm->prepareForm();

            // if the form was submitted, try to save user input
            $result = $htmlForm->tryAuthorizedSubmit();

            // if the settings were succesfully saved, inform the user
            if (isset($wgCryptoPhotoConfig['save_success']) && $wgCryptoPhotoConfig['save_success'])
                $out->addHTML('<div class="successbox">' . wfMessage('cpsettings-saved-success')->text() . '</div>');

            // add the form to HTML output
            $htmlForm->displayForm($result);

            // get server IP, this is used for the CryptoPhoto autoconfiguration
            $svip = file_get_contents('http://cryptophoto.com/show_my_ip');

            // CryptoPhoto autoconfiguration wizzard and test function support
            $out->addHTML(<<<EOT
<script language="javascript" type="text/javascript">
 
if (typeof CPPluginWizard !== 'object') CPPluginWizard = {};

CPPluginWizard.serverip = "$svip";
CPPluginWizard.callback = function(keys) {

  if(keys.PRIVATE_KEY && keys.PUBLIC_KEY) {
    document.getElementById("publickey").value = keys.PUBLIC_KEY;
    document.getElementById("privatekey").value = keys.PRIVATE_KEY;
    if (!!!document.getElementById("salt").value) {
      document.getElementById("salt").value = guid();
    }
    //testConfig();
  }

};

//Browser Support Code
function testConfig() {

  var ajaxRequest;  // The variable that makes Ajax possible!
  
  try {
    // Opera 8.0+, Firefox, Safari
    ajaxRequest = new XMLHttpRequest();
  } catch (e) {
    // Internet Explorer Browsers
    try {
      ajaxRequest = new ActiveXObject("Msxml2.XMLHTTP");
    } catch (e) {
      try {
        ajaxRequest = new ActiveXObject("Microsoft.XMLHTTP");
      } catch (e) {
        // Something went wrong
        return false;
      }
    }
  }
  
  
  // Create a function that will receive data sent from the server
  ajaxRequest.onreadystatechange = function() {
  
    if(ajaxRequest.readyState == 4 && ajaxRequest.status == 200) {
      document.getElementById("testresult").style.display = "block";
      document.getElementById("testresult").innerHTML=ajaxRequest.responseText;

      if(ajaxRequest.responseText.indexOf('success') > -1) {
        document.getElementById("testresult").className="successbox";
        document.getElementById("testresult").innerHTML="Cryptophoto plugin is configured properly. Don't forget to save your settings.";
        document.getElementById("testresult").setAttribute("style","color:green"); 
      }
      else if(ajaxRequest.responseText.indexOf('timestamp') > -1) {
        document.getElementById("testresult").className="errorbox";
        document.getElementById("testresult").innerHTML="Error: Please verify that the date/time is properly set on this server.";
        document.getElementById("testresult").setAttribute("style","color:red"); 
      }
      else if(ajaxRequest.responseText.indexOf('ip') > -1) {
        var ip = "";
        ip = ajaxRequest.responseText.replace("ip ","");

        if (ip) { 
          ip = " (" + ip +")";
        } 
        else {
          ip = "";
        }

        document.getElementById("testresult").className="errorbox";
        document.getElementById("testresult").innerHTML="Error: The IP of this server"+ip+" is not added to the list of allowed IPs. Check your settings in your CryptoPhoto.com account.";
        document.getElementById("testresult").setAttribute("style","color:red"); 
      }
      else if(ajaxRequest.responseText.indexOf('invalid') > -1) {
        document.getElementById("testresult").className="errorbox";
        document.getElementById("testresult").innerHTML="Error: Please verify that the API keys are valid.";
        document.getElementById("testresult").setAttribute("style","color:red"); 
      }
      else if(ajaxRequest.responseText.indexOf('empty') > -1) {
        document.getElementById("testresult").className="errorbox";
        document.getElementById("testresult").innerHTML="Error: Please configure and save your API keys and salt.";
        document.getElementById("testresult").setAttribute("style","color:red"); 
      } else {
        document.getElementById("testresult").className="errorbox";
        document.getElementById("testresult").innerHTML="Error: Unknown error. Please check your CryptoPhoto server setting.";
        document.getElementById("testresult").setAttribute("style","color:red"); 
      }
    }
  };

  var data = "test=1";
  data += "&publickey=" + encodeURIComponent(document.getElementById("publickey").value);
  data += "&privatekey=" + encodeURIComponent(document.getElementById("privatekey").value);
  data += "&salt=" + encodeURIComponent(document.getElementById("salt").value);
  data += "&server=" + encodeURIComponent(document.getElementById("server").value);
  
  ajaxRequest.open("POST", location.protocol+"//"+location.host+location.pathname, true);
  ajaxRequest.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
  ajaxRequest.send(data); 
}

function guid() {
  var chars = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'.split(''); 
  var uuid = new Array(36), rnd=0, r;
  for (var i = 0; i < 36; i++) {
    if (i==8 || i==13 ||  i==18 || i==23) {
      uuid[i] = '';
    } else if (i==14) {
      uuid[i] = '4';
    } else {
      if (rnd <= 0x02) rnd = 0x2000000 + (Math.random()*0x1000000)|0;
      r = rnd & 0xf;
      rnd = rnd >> 4;
      uuid[i] = chars[(i == 19) ? (r & 0x3) | 0x8 : r];
    }
  }
  return uuid.join('');
}

function add_script (a) {
  var b = document.createElement("script");
  b.type = "text/javascript";
  b.src = a;
  document.body.appendChild(b);
}

var server = document.getElementById("server").value;
if(server.indexOf("http") > -1) {
  add_script(server+"/api/plugin/config.js");
}

</script>
EOT
            );
        } else {
            // attempt to reach the CryptoPhoto settings page, but without administrator rights
            $wgOut->addWikiText(wfMessage('cp-error-not-admin')->text());
        }
    }
    
    /**
    * Save CryptoPhoto settings from user input
    */
    public function save($formData) {
        global $wgCryptoPhotoConfig;

        // try to save the settings in the database
        if (CryptoPhotoDatabaseSupport::saveSettings(
            $formData['publickey'],
            $formData['privatekey'],
            $formData['salt'],
            $formData['server'],
            (isset($formData['mandatory_groups']) ? $formData['mandatory_groups'] : array()),
            (isset($formData['groups']) ? $formData['groups'] : array())
           )) {
            // save sucessful 
            $wgCryptoPhotoConfig['save_success'] = 1;
            
            // we return false so the HTMLForm output will still be displayed
            return false;
        };
        
        // there was some kind of error
        return wfMessage('cpsettings-saved-error')->text();
    }

    /**
    * Test CryptoPhoto setup
    */
    public function test() {
        // get the WebRequest object
        $req = $this->getRequest();

        // get CryptoPhoto setup from user input
        $privatekey = $req->getVal('privatekey');
        $publickey = $req->getVal('publickey');
        $salt = $req->getVal('salt');
        $server = $req->getVal('server');
    
        // test result
        $result = "";
        
        // setup server if needed
        if (!$server) $server = "https://cryptophoto.com";
    
        // test non-empty input
        if ($privatekey && $publickey && $salt) {
            // new CryptoPhoto instance
            $cp = new CryptoPhotoUtils($server, $privatekey, $publickey, md5($salt));
            
            // attempt to start a new session
            $session = $cp->start_session($_SERVER['REMOTE_ADDR']);

            // test if new session started successfully
            if ($session[0] == TRUE) {
                $result = "success";
            } else {				
                // something went wrong, look for the reason why
                if ($session[1] == "incorrect-timestamp") {
                    $result = "timestamp";
                } elseif (strpos($session[1],"ip-not-allowed") !== false) {
                    $result = "ip " . $session[2];
                } else if ($session[1] == "incorrect-signature" || $session[1] == "incorrect-api-request") {
                    $result = "invalid";
                } else $result = "other";
            }
        } else {
            // one or more empty fields
            $result = "empty";
        }

        // disable MediaWiki output (our AJAX test request expects a simple text)
        $this->getOutput()->disable();
        
        // send back the result
        header("Content-type: text/plain");
        print $result;
        
        return true;
    }
}

?>
